/*******************************************************************************
* File Name: SER.c
* Version 3.0
*
* Description:
*  This file provides the source code to the API for the SCB Component.
*
* Note:
*
*******************************************************************************
* Copyright 2013-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SER_PVT.h"

#if (SER_SCB_MODE_I2C_INC)
    #include "SER_I2C_PVT.h"
#endif /* (SER_SCB_MODE_I2C_INC) */

#if (SER_SCB_MODE_EZI2C_INC)
    #include "SER_EZI2C_PVT.h"
#endif /* (SER_SCB_MODE_EZI2C_INC) */

#if (SER_SCB_MODE_SPI_INC || SER_SCB_MODE_UART_INC)
    #include "SER_SPI_UART_PVT.h"
#endif /* (SER_SCB_MODE_SPI_INC || SER_SCB_MODE_UART_INC) */


/***************************************
*    Run Time Configuration Vars
***************************************/

/* Stores internal component configuration for Unconfigured mode */
#if (SER_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Common configuration variables */
    uint8 SER_scbMode = SER_SCB_MODE_UNCONFIG;
    uint8 SER_scbEnableWake;
    uint8 SER_scbEnableIntr;

    /* I2C configuration variables */
    uint8 SER_mode;
    uint8 SER_acceptAddr;

    /* SPI/UART configuration variables */
    volatile uint8 * SER_rxBuffer;
    uint8  SER_rxDataBits;
    uint32 SER_rxBufferSize;

    volatile uint8 * SER_txBuffer;
    uint8  SER_txDataBits;
    uint32 SER_txBufferSize;

    /* EZI2C configuration variables */
    uint8 SER_numberOfAddr;
    uint8 SER_subAddrSize;
#endif /* (SER_SCB_MODE_UNCONFIG_CONST_CFG) */


/***************************************
*     Common SCB Vars
***************************************/

uint8 SER_initVar = 0u;

#if (SER_SCB_IRQ_INTERNAL)
#if !defined (CY_REMOVE_SER_CUSTOM_INTR_HANDLER)
    void (*SER_customIntrHandler)(void) = NULL;
#endif /* !defined (CY_REMOVE_SER_CUSTOM_INTR_HANDLER) */
#endif /* (SER_SCB_IRQ_INTERNAL) */


/***************************************
*    Private Function Prototypes
***************************************/

static void SER_ScbEnableIntr(void);
static void SER_ScbModeStop(void);
static void SER_ScbModePostEnable(void);


/*******************************************************************************
* Function Name: SER_Init
********************************************************************************
*
* Summary:
*  Initializes the SCB component to operate in one of the selected
*  configurations: I2C, SPI, UART or EZI2C.
*  When the configuration is set to "Unconfigured SCB", this function does
*  not do any initialization. Use mode-specific initialization APIs instead:
*  SCB_I2CInit, SCB_SpiInit, SCB_UartInit or SCB_EzI2CInit.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SER_Init(void)
{
#if (SER_SCB_MODE_UNCONFIG_CONST_CFG)
    if (SER_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        SER_initVar = 0u;
    }
    else
    {
        /* Initialization was done before this function call */
    }

#elif (SER_SCB_MODE_I2C_CONST_CFG)
    SER_I2CInit();

#elif (SER_SCB_MODE_SPI_CONST_CFG)
    SER_SpiInit();

#elif (SER_SCB_MODE_UART_CONST_CFG)
    SER_UartInit();

#elif (SER_SCB_MODE_EZI2C_CONST_CFG)
    SER_EzI2CInit();

#endif /* (SER_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SER_Enable
********************************************************************************
*
* Summary:
*  Enables the SCB component operation.
*  The SCB configuration should be not changed when the component is enabled.
*  Any configuration changes should be made after disabling the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SER_Enable(void)
{
#if (SER_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Enable SCB block, only if it is already configured */
    if (!SER_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        SER_CTRL_REG |= SER_CTRL_ENABLED;

        SER_ScbEnableIntr();

        /* Call PostEnable function specific to current operation mode */
        SER_ScbModePostEnable();
    }
#else
    SER_CTRL_REG |= SER_CTRL_ENABLED;

    SER_ScbEnableIntr();

    /* Call PostEnable function specific to current operation mode */
    SER_ScbModePostEnable();
#endif /* (SER_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SER_Start
********************************************************************************
*
* Summary:
*  Invokes SCB_Init() and SCB_Enable().
*  After this function call, the component is enabled and ready for operation.
*  When configuration is set to "Unconfigured SCB", the component must first be
*  initialized to operate in one of the following configurations: I2C, SPI, UART
*  or EZ I2C. Otherwise this function does not enable the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  SER_initVar - used to check initial configuration, modified
*  on first function call.
*
*******************************************************************************/
void SER_Start(void)
{
    if (0u == SER_initVar)
    {
        SER_Init();
        SER_initVar = 1u; /* Component was initialized */
    }

    SER_Enable();
}


/*******************************************************************************
* Function Name: SER_Stop
********************************************************************************
*
* Summary:
*  Disables the SCB component and its interrupt.
*  It also disables all TX interrupt sources so as not to cause an unexpected
*  interrupt trigger because after the component is enabled, the TX FIFO 
*  is empty.
*
* Parameters:
*  None
*
* Return:
*  None
* 
*******************************************************************************/
void SER_Stop(void)
{
#if (SER_SCB_IRQ_INTERNAL)
    SER_DisableInt();
#endif /* (SER_SCB_IRQ_INTERNAL) */

    /* Call Stop function specific to current operation mode */
    SER_ScbModeStop();

    /* Disable SCB IP */
    SER_CTRL_REG &= (uint32) ~SER_CTRL_ENABLED;

    /* Disable all TX interrupt sources so as not to cause an unexpected
    * interrupt trigger because after the component is enabled, the TX FIFO
    * is empty.
    * For SCB IP v0, it is critical as it does not mask-out interrupt
    * sources when they are disabled. This can cause a code lock-up in the
    * interrupt handler because TX FIFO cannot be loaded after the block
    * is disabled.
    */
    SER_SetTxInterruptMode(SER_NO_INTR_SOURCES);

#if (SER_SCB_IRQ_INTERNAL)
    SER_ClearPendingInt();
#endif /* (SER_SCB_IRQ_INTERNAL) */
}


/*******************************************************************************
* Function Name: SER_SetRxFifoLevel
********************************************************************************
*
* Summary:
*  Sets level in the RX FIFO to generate a RX level interrupt.
*  When the RX FIFO has more entries than the RX FIFO level an RX level
*  interrupt request is generated.
*
* Parameters:
*  level: Level in the RX FIFO to generate RX level interrupt.
*         The range of valid level values is between 0 and RX FIFO depth - 1.
*
* Return:
*  None
*
*******************************************************************************/
void SER_SetRxFifoLevel(uint32 level)
{
    uint32 rxFifoCtrl;

    rxFifoCtrl = SER_RX_FIFO_CTRL_REG;

    rxFifoCtrl &= ((uint32) ~SER_RX_FIFO_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
    rxFifoCtrl |= ((uint32) (SER_RX_FIFO_CTRL_TRIGGER_LEVEL_MASK & level));

    SER_RX_FIFO_CTRL_REG = rxFifoCtrl;
}


/*******************************************************************************
* Function Name: SER_SetTxFifoLevel
********************************************************************************
*
* Summary:
*  Sets level in the TX FIFO to generate a TX level interrupt.
*  When the TX FIFO has more entries than the TX FIFO level an TX level
*  interrupt request is generated.
*
* Parameters:
*  level: Level in the TX FIFO to generate TX level interrupt.
*         The range of valid level values is between 0 and TX FIFO depth - 1.
*
* Return:
*  None
*
*******************************************************************************/
void SER_SetTxFifoLevel(uint32 level)
{
    uint32 txFifoCtrl;

    txFifoCtrl = SER_TX_FIFO_CTRL_REG;

    txFifoCtrl &= ((uint32) ~SER_TX_FIFO_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
    txFifoCtrl |= ((uint32) (SER_TX_FIFO_CTRL_TRIGGER_LEVEL_MASK & level));

    SER_TX_FIFO_CTRL_REG = txFifoCtrl;
}


#if (SER_SCB_IRQ_INTERNAL)
    /*******************************************************************************
    * Function Name: SER_SetCustomInterruptHandler
    ********************************************************************************
    *
    * Summary:
    *  Registers a function to be called by the internal interrupt handler.
    *  First the function that is registered is called, then the internal interrupt
    *  handler performs any operation such as software buffer management functions
    *  before the interrupt returns.  It is the user's responsibility not to break
    *  the software buffer operations. Only one custom handler is supported, which
    *  is the function provided by the most recent call.
    *  At the initialization time no custom handler is registered.
    *
    * Parameters:
    *  func: Pointer to the function to register.
    *        The value NULL indicates to remove the current custom interrupt
    *        handler.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SER_SetCustomInterruptHandler(void (*func)(void))
    {
    #if !defined (CY_REMOVE_SER_CUSTOM_INTR_HANDLER)
        SER_customIntrHandler = func; /* Register interrupt handler */
    #else
        if (NULL != func)
        {
            /* Suppress compiler warning */
        }
    #endif /* !defined (CY_REMOVE_SER_CUSTOM_INTR_HANDLER) */
    }
#endif /* (SER_SCB_IRQ_INTERNAL) */


/*******************************************************************************
* Function Name: SER_ScbModeEnableIntr
********************************************************************************
*
* Summary:
*  Enables an interrupt for a specific mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
static void SER_ScbEnableIntr(void)
{
#if (SER_SCB_IRQ_INTERNAL)
    /* Enable interrupt in NVIC */
    #if (SER_SCB_MODE_UNCONFIG_CONST_CFG)
        if (0u != SER_scbEnableIntr)
        {
            SER_EnableInt();
        }

    #else
        SER_EnableInt();

    #endif /* (SER_SCB_MODE_UNCONFIG_CONST_CFG) */
#endif /* (SER_SCB_IRQ_INTERNAL) */
}


/*******************************************************************************
* Function Name: SER_ScbModePostEnable
********************************************************************************
*
* Summary:
*  Calls the PostEnable function for a specific operation mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
static void SER_ScbModePostEnable(void)
{
#if (SER_SCB_MODE_UNCONFIG_CONST_CFG)
#if (!SER_CY_SCBIP_V1)
    if (SER_SCB_MODE_SPI_RUNTM_CFG)
    {
        SER_SpiPostEnable();
    }
    else if (SER_SCB_MODE_UART_RUNTM_CFG)
    {
        SER_UartPostEnable();
    }
    else
    {
        /* Unknown mode: do nothing */
    }
#endif /* (!SER_CY_SCBIP_V1) */

#elif (SER_SCB_MODE_SPI_CONST_CFG)
    SER_SpiPostEnable();

#elif (SER_SCB_MODE_UART_CONST_CFG)
    SER_UartPostEnable();

#else
    /* Unknown mode: do nothing */
#endif /* (SER_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SER_ScbModeStop
********************************************************************************
*
* Summary:
*  Calls the Stop function for a specific operation mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
static void SER_ScbModeStop(void)
{
#if (SER_SCB_MODE_UNCONFIG_CONST_CFG)
    if (SER_SCB_MODE_I2C_RUNTM_CFG)
    {
        SER_I2CStop();
    }
    else if (SER_SCB_MODE_EZI2C_RUNTM_CFG)
    {
        SER_EzI2CStop();
    }
#if (!SER_CY_SCBIP_V1)
    else if (SER_SCB_MODE_SPI_RUNTM_CFG)
    {
        SER_SpiStop();
    }
    else if (SER_SCB_MODE_UART_RUNTM_CFG)
    {
        SER_UartStop();
    }
#endif /* (!SER_CY_SCBIP_V1) */
    else
    {
        /* Unknown mode: do nothing */
    }
#elif (SER_SCB_MODE_I2C_CONST_CFG)
    SER_I2CStop();

#elif (SER_SCB_MODE_EZI2C_CONST_CFG)
    SER_EzI2CStop();

#elif (SER_SCB_MODE_SPI_CONST_CFG)
    SER_SpiStop();

#elif (SER_SCB_MODE_UART_CONST_CFG)
    SER_UartStop();

#else
    /* Unknown mode: do nothing */
#endif /* (SER_SCB_MODE_UNCONFIG_CONST_CFG) */
}


#if (SER_SCB_MODE_UNCONFIG_CONST_CFG)
    /*******************************************************************************
    * Function Name: SER_SetPins
    ********************************************************************************
    *
    * Summary:
    *  Sets the pins settings accordingly to the selected operation mode.
    *  Only available in the Unconfigured operation mode. The mode specific
    *  initialization function calls it.
    *  Pins configuration is set by PSoC Creator when a specific mode of operation
    *  is selected in design time.
    *
    * Parameters:
    *  mode:      Mode of SCB operation.
    *  subMode:   Sub-mode of SCB operation. It is only required for SPI and UART
    *             modes.
    *  uartEnableMask: enables TX or RX direction and RTS and CTS signals.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SER_SetPins(uint32 mode, uint32 subMode, uint32 uartEnableMask)
    {
        uint32 hsiomSel [SER_SCB_PINS_NUMBER];
        uint32 pinsDm   [SER_SCB_PINS_NUMBER];

    #if (!SER_CY_SCBIP_V1)
        uint32 pinsInBuf = 0u;
    #endif /* (!SER_CY_SCBIP_V1) */

        uint32 i;

        /* Set default HSIOM to GPIO and Drive Mode to Analog Hi-Z */
        for (i = 0u; i < SER_SCB_PINS_NUMBER; i++)
        {
            hsiomSel[i]  = SER_HSIOM_DEF_SEL;
            pinsDm[i]    = SER_PIN_DM_ALG_HIZ;
        }

        if ((SER_SCB_MODE_I2C   == mode) ||
           (SER_SCB_MODE_EZI2C == mode))
        {
            hsiomSel[SER_RX_SCL_MOSI_PIN_INDEX] = SER_HSIOM_I2C_SEL;
            hsiomSel[SER_TX_SDA_MISO_PIN_INDEX] = SER_HSIOM_I2C_SEL;

            pinsDm[SER_RX_SCL_MOSI_PIN_INDEX] = SER_PIN_DM_OD_LO;
            pinsDm[SER_TX_SDA_MISO_PIN_INDEX] = SER_PIN_DM_OD_LO;
        }
    #if (!SER_CY_SCBIP_V1)
        else if (SER_SCB_MODE_SPI == mode)
        {
            hsiomSel[SER_RX_SCL_MOSI_PIN_INDEX] = SER_HSIOM_SPI_SEL;
            hsiomSel[SER_TX_SDA_MISO_PIN_INDEX] = SER_HSIOM_SPI_SEL;
            hsiomSel[SER_SCLK_PIN_INDEX] = SER_HSIOM_SPI_SEL;

            if (SER_SPI_SLAVE == subMode)
            {
                /* Slave */
                pinsDm[SER_RX_SCL_MOSI_PIN_INDEX] = SER_PIN_DM_DIG_HIZ;
                pinsDm[SER_TX_SDA_MISO_PIN_INDEX] = SER_PIN_DM_STRONG;
                pinsDm[SER_SCLK_PIN_INDEX] = SER_PIN_DM_DIG_HIZ;

            #if (SER_SS0_PIN)
                /* Only SS0 is valid choice for Slave */
                hsiomSel[SER_SS0_PIN_INDEX] = SER_HSIOM_SPI_SEL;
                pinsDm  [SER_SS0_PIN_INDEX] = SER_PIN_DM_DIG_HIZ;
            #endif /* (SER_SS0_PIN) */

            #if (SER_TX_SDA_MISO_PIN)
                /* Disable input buffer */
                 pinsInBuf |= SER_TX_SDA_MISO_PIN_MASK;
            #endif /* (SER_TX_SDA_MISO_PIN) */
            }
            else /* (Master) */
            {
                pinsDm[SER_RX_SCL_MOSI_PIN_INDEX] = SER_PIN_DM_STRONG;
                pinsDm[SER_TX_SDA_MISO_PIN_INDEX] = SER_PIN_DM_DIG_HIZ;
                pinsDm[SER_SCLK_PIN_INDEX] = SER_PIN_DM_STRONG;

            #if (SER_SS0_PIN)
                hsiomSel [SER_SS0_PIN_INDEX] = SER_HSIOM_SPI_SEL;
                pinsDm   [SER_SS0_PIN_INDEX] = SER_PIN_DM_STRONG;
                pinsInBuf |= SER_SS0_PIN_MASK;
            #endif /* (SER_SS0_PIN) */

            #if (SER_SS1_PIN)
                hsiomSel [SER_SS1_PIN_INDEX] = SER_HSIOM_SPI_SEL;
                pinsDm   [SER_SS1_PIN_INDEX] = SER_PIN_DM_STRONG;
                pinsInBuf |= SER_SS1_PIN_MASK;
            #endif /* (SER_SS1_PIN) */

            #if (SER_SS2_PIN)
                hsiomSel [SER_SS2_PIN_INDEX] = SER_HSIOM_SPI_SEL;
                pinsDm   [SER_SS2_PIN_INDEX] = SER_PIN_DM_STRONG;
                pinsInBuf |= SER_SS2_PIN_MASK;
            #endif /* (SER_SS2_PIN) */

            #if (SER_SS3_PIN)
                hsiomSel [SER_SS3_PIN_INDEX] = SER_HSIOM_SPI_SEL;
                pinsDm   [SER_SS3_PIN_INDEX] = SER_PIN_DM_STRONG;
                pinsInBuf |= SER_SS3_PIN_MASK;
            #endif /* (SER_SS3_PIN) */

                /* Disable input buffers */
            #if (SER_RX_SCL_MOSI_PIN)
                pinsInBuf |= SER_RX_SCL_MOSI_PIN_MASK;
            #endif /* (SER_RX_SCL_MOSI_PIN) */

             #if (SER_RX_WAKE_SCL_MOSI_PIN)
                pinsInBuf |= SER_RX_WAKE_SCL_MOSI_PIN_MASK;
            #endif /* (SER_RX_WAKE_SCL_MOSI_PIN) */

            #if (SER_SCLK_PIN)
                pinsInBuf |= SER_SCLK_PIN_MASK;
            #endif /* (SER_SCLK_PIN) */
            }
        }
        else /* UART */
        {
            if (SER_UART_MODE_SMARTCARD == subMode)
            {
                /* SmartCard */
                hsiomSel[SER_TX_SDA_MISO_PIN_INDEX] = SER_HSIOM_UART_SEL;
                pinsDm  [SER_TX_SDA_MISO_PIN_INDEX] = SER_PIN_DM_OD_LO;
            }
            else /* Standard or IrDA */
            {
                if (0u != (SER_UART_RX_PIN_ENABLE & uartEnableMask))
                {
                    hsiomSel[SER_RX_SCL_MOSI_PIN_INDEX] = SER_HSIOM_UART_SEL;
                    pinsDm  [SER_RX_SCL_MOSI_PIN_INDEX] = SER_PIN_DM_DIG_HIZ;
                }

                if (0u != (SER_UART_TX_PIN_ENABLE & uartEnableMask))
                {
                    hsiomSel[SER_TX_SDA_MISO_PIN_INDEX] = SER_HSIOM_UART_SEL;
                    pinsDm  [SER_TX_SDA_MISO_PIN_INDEX] = SER_PIN_DM_STRONG;

                #if (SER_TX_SDA_MISO_PIN)
                     pinsInBuf |= SER_TX_SDA_MISO_PIN_MASK;
                #endif /* (SER_TX_SDA_MISO_PIN) */
                }

            #if !(SER_CY_SCBIP_V0 || SER_CY_SCBIP_V1)
                if (SER_UART_MODE_STD == subMode)
                {
                    if (0u != (SER_UART_CTS_PIN_ENABLE & uartEnableMask))
                    {
                        /* CTS input is multiplexed with SCLK */
                        hsiomSel[SER_SCLK_PIN_INDEX] = SER_HSIOM_UART_SEL;
                        pinsDm  [SER_SCLK_PIN_INDEX] = SER_PIN_DM_DIG_HIZ;
                    }

                    if (0u != (SER_UART_RTS_PIN_ENABLE & uartEnableMask))
                    {
                        /* RTS output is multiplexed with SS0 */
                        hsiomSel[SER_SS0_PIN_INDEX] = SER_HSIOM_UART_SEL;
                        pinsDm  [SER_SS0_PIN_INDEX] = SER_PIN_DM_STRONG;

                    #if (SER_SS0_PIN)
                        /* Disable input buffer */
                        pinsInBuf |= SER_SS0_PIN_MASK;
                    #endif /* (SER_SS0_PIN) */
                    }
                }
            #endif /* !(SER_CY_SCBIP_V0 || SER_CY_SCBIP_V1) */
            }
        }
    #endif /* (!SER_CY_SCBIP_V1) */

    /* Configure pins: set HSIOM, DM and InputBufEnable */
    /* Note: the DR register settings do not effect the pin output if HSIOM is other than GPIO */

    #if (SER_RX_WAKE_SCL_MOSI_PIN)
        SER_SET_HSIOM_SEL(SER_RX_WAKE_SCL_MOSI_HSIOM_REG,
                                       SER_RX_WAKE_SCL_MOSI_HSIOM_MASK,
                                       SER_RX_WAKE_SCL_MOSI_HSIOM_POS,
                                       hsiomSel[SER_RX_WAKE_SCL_MOSI_PIN_INDEX]);

        SER_uart_rx_wake_i2c_scl_spi_mosi_SetDriveMode((uint8)
                                                               pinsDm[SER_RX_WAKE_SCL_MOSI_PIN_INDEX]);

        SER_SET_INP_DIS(SER_uart_rx_wake_i2c_scl_spi_mosi_INP_DIS,
                                     SER_uart_rx_wake_i2c_scl_spi_mosi_MASK,
                                     (0u != (pinsInBuf & SER_RX_WAKE_SCL_MOSI_PIN_MASK)));

         /* Set interrupt on falling edge */
        SER_SET_INCFG_TYPE(SER_RX_WAKE_SCL_MOSI_INTCFG_REG,
                                        SER_RX_WAKE_SCL_MOSI_INTCFG_TYPE_MASK,
                                        SER_RX_WAKE_SCL_MOSI_INTCFG_TYPE_POS,
                                        SER_INTCFG_TYPE_FALLING_EDGE);
    #endif /* (SER_RX_WAKE_SCL_MOSI_PIN) */

    #if (SER_RX_SCL_MOSI_PIN)
        SER_SET_HSIOM_SEL(SER_RX_SCL_MOSI_HSIOM_REG,
                                       SER_RX_SCL_MOSI_HSIOM_MASK,
                                       SER_RX_SCL_MOSI_HSIOM_POS,
                                        hsiomSel[SER_RX_SCL_MOSI_PIN_INDEX]);

        SER_uart_rx_i2c_scl_spi_mosi_SetDriveMode((uint8) pinsDm[SER_RX_SCL_MOSI_PIN_INDEX]);

    #if (!SER_CY_SCBIP_V1)
        SER_SET_INP_DIS(SER_uart_rx_i2c_scl_spi_mosi_INP_DIS,
                                     SER_uart_rx_i2c_scl_spi_mosi_MASK,
                                     (0u != (pinsInBuf & SER_RX_SCL_MOSI_PIN_MASK)));
    #endif /* (!SER_CY_SCBIP_V1) */
    #endif /* (SER_RX_SCL_MOSI_PIN) */

    #if (SER_TX_SDA_MISO_PIN)
        SER_SET_HSIOM_SEL(SER_TX_SDA_MISO_HSIOM_REG,
                                       SER_TX_SDA_MISO_HSIOM_MASK,
                                       SER_TX_SDA_MISO_HSIOM_POS,
                                        hsiomSel[SER_TX_SDA_MISO_PIN_INDEX]);

        SER_uart_tx_i2c_sda_spi_miso_SetDriveMode((uint8) pinsDm[SER_TX_SDA_MISO_PIN_INDEX]);

    #if (!SER_CY_SCBIP_V1)
        SER_SET_INP_DIS(SER_uart_tx_i2c_sda_spi_miso_INP_DIS,
                                     SER_uart_tx_i2c_sda_spi_miso_MASK,
                                    (0u != (pinsInBuf & SER_TX_SDA_MISO_PIN_MASK)));
    #endif /* (!SER_CY_SCBIP_V1) */
    #endif /* (SER_RX_SCL_MOSI_PIN) */

    #if (SER_SCLK_PIN)
        SER_SET_HSIOM_SEL(SER_SCLK_HSIOM_REG,
                                       SER_SCLK_HSIOM_MASK,
                                       SER_SCLK_HSIOM_POS,
                                       hsiomSel[SER_SCLK_PIN_INDEX]);

        SER_spi_sclk_SetDriveMode((uint8) pinsDm[SER_SCLK_PIN_INDEX]);

        SER_SET_INP_DIS(SER_spi_sclk_INP_DIS,
                                     SER_spi_sclk_MASK,
                                     (0u != (pinsInBuf & SER_SCLK_PIN_MASK)));
    #endif /* (SER_SCLK_PIN) */

    #if (SER_SS0_PIN)
        SER_SET_HSIOM_SEL(SER_SS0_HSIOM_REG,
                                       SER_SS0_HSIOM_MASK,
                                       SER_SS0_HSIOM_POS,
                                       hsiomSel[SER_SS0_PIN_INDEX]);

        SER_spi_ss0_SetDriveMode((uint8) pinsDm[SER_SS0_PIN_INDEX]);

        SER_SET_INP_DIS(SER_spi_ss0_INP_DIS,
                                     SER_spi_ss0_MASK,
                                     (0u != (pinsInBuf & SER_SS0_PIN_MASK)));
    #endif /* (SER_SS0_PIN) */

    #if (SER_SS1_PIN)
        SER_SET_HSIOM_SEL(SER_SS1_HSIOM_REG,
                                       SER_SS1_HSIOM_MASK,
                                       SER_SS1_HSIOM_POS,
                                       hsiomSel[SER_SS1_PIN_INDEX]);

        SER_spi_ss1_SetDriveMode((uint8) pinsDm[SER_SS1_PIN_INDEX]);

        SER_SET_INP_DIS(SER_spi_ss1_INP_DIS,
                                     SER_spi_ss1_MASK,
                                     (0u != (pinsInBuf & SER_SS1_PIN_MASK)));
    #endif /* (SER_SS1_PIN) */

    #if (SER_SS2_PIN)
        SER_SET_HSIOM_SEL(SER_SS2_HSIOM_REG,
                                       SER_SS2_HSIOM_MASK,
                                       SER_SS2_HSIOM_POS,
                                       hsiomSel[SER_SS2_PIN_INDEX]);

        SER_spi_ss2_SetDriveMode((uint8) pinsDm[SER_SS2_PIN_INDEX]);

        SER_SET_INP_DIS(SER_spi_ss2_INP_DIS,
                                     SER_spi_ss2_MASK,
                                     (0u != (pinsInBuf & SER_SS2_PIN_MASK)));
    #endif /* (SER_SS2_PIN) */

    #if (SER_SS3_PIN)
        SER_SET_HSIOM_SEL(SER_SS3_HSIOM_REG,
                                       SER_SS3_HSIOM_MASK,
                                       SER_SS3_HSIOM_POS,
                                       hsiomSel[SER_SS3_PIN_INDEX]);

        SER_spi_ss3_SetDriveMode((uint8) pinsDm[SER_SS3_PIN_INDEX]);

        SER_SET_INP_DIS(SER_spi_ss3_INP_DIS,
                                     SER_spi_ss3_MASK,
                                     (0u != (pinsInBuf & SER_SS3_PIN_MASK)));
    #endif /* (SER_SS3_PIN) */
    }

#endif /* (SER_SCB_MODE_UNCONFIG_CONST_CFG) */


#if (SER_CY_SCBIP_V0 || SER_CY_SCBIP_V1)
    /*******************************************************************************
    * Function Name: SER_I2CSlaveNackGeneration
    ********************************************************************************
    *
    * Summary:
    *  Sets command to generate NACK to the address or data.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SER_I2CSlaveNackGeneration(void)
    {
        /* Check for EC_AM toggle condition: EC_AM and clock stretching for address are enabled */
        if ((0u != (SER_CTRL_REG & SER_CTRL_EC_AM_MODE)) &&
            (0u == (SER_I2C_CTRL_REG & SER_I2C_CTRL_S_NOT_READY_ADDR_NACK)))
        {
            /* Toggle EC_AM before NACK generation */
            SER_CTRL_REG &= ~SER_CTRL_EC_AM_MODE;
            SER_CTRL_REG |=  SER_CTRL_EC_AM_MODE;
        }

        SER_I2C_SLAVE_CMD_REG = SER_I2C_SLAVE_CMD_S_NACK;
    }
#endif /* (SER_CY_SCBIP_V0 || SER_CY_SCBIP_V1) */


/* [] END OF FILE */
