/*******************************************************************************
* File Name: SER_PM.c
* Version 3.0
*
* Description:
*  This file provides the source code to the Power Management support for
*  the SCB Component.
*
* Note:
*
********************************************************************************
* Copyright 2013-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SER.h"
#include "SER_PVT.h"

#if(SER_SCB_MODE_I2C_INC)
    #include "SER_I2C_PVT.h"
#endif /* (SER_SCB_MODE_I2C_INC) */

#if(SER_SCB_MODE_EZI2C_INC)
    #include "SER_EZI2C_PVT.h"
#endif /* (SER_SCB_MODE_EZI2C_INC) */

#if(SER_SCB_MODE_SPI_INC || SER_SCB_MODE_UART_INC)
    #include "SER_SPI_UART_PVT.h"
#endif /* (SER_SCB_MODE_SPI_INC || SER_SCB_MODE_UART_INC) */


/***************************************
*   Backup Structure declaration
***************************************/

#if(SER_SCB_MODE_UNCONFIG_CONST_CFG || \
   (SER_SCB_MODE_I2C_CONST_CFG   && (!SER_I2C_WAKE_ENABLE_CONST))   || \
   (SER_SCB_MODE_EZI2C_CONST_CFG && (!SER_EZI2C_WAKE_ENABLE_CONST)) || \
   (SER_SCB_MODE_SPI_CONST_CFG   && (!SER_SPI_WAKE_ENABLE_CONST))   || \
   (SER_SCB_MODE_UART_CONST_CFG  && (!SER_UART_WAKE_ENABLE_CONST)))

    SER_BACKUP_STRUCT SER_backup =
    {
        0u, /* enableState */
    };
#endif


/*******************************************************************************
* Function Name: SER_Sleep
********************************************************************************
*
* Summary:
*  Prepares the component to enter Deep Sleep.
*  The "Enable wakeup from Sleep Mode" selection has an influence on
*  this function implementation.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SER_Sleep(void)
{
#if(SER_SCB_MODE_UNCONFIG_CONST_CFG)

    if(SER_SCB_WAKE_ENABLE_CHECK)
    {
        if(SER_SCB_MODE_I2C_RUNTM_CFG)
        {
            SER_I2CSaveConfig();
        }
        else if(SER_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            SER_EzI2CSaveConfig();
        }
    #if(!SER_CY_SCBIP_V1)
        else if(SER_SCB_MODE_SPI_RUNTM_CFG)
        {
            SER_SpiSaveConfig();
        }
        else if(SER_SCB_MODE_UART_RUNTM_CFG)
        {
            SER_UartSaveConfig();
        }
    #endif /* (!SER_CY_SCBIP_V1) */
        else
        {
            /* Unknown mode */
        }
    }
    else
    {
        SER_backup.enableState = (uint8) SER_GET_CTRL_ENABLED;

        if(0u != SER_backup.enableState)
        {
            SER_Stop();
        }
    }

#else

    #if (SER_SCB_MODE_I2C_CONST_CFG && SER_I2C_WAKE_ENABLE_CONST)
        SER_I2CSaveConfig();

    #elif (SER_SCB_MODE_EZI2C_CONST_CFG && SER_EZI2C_WAKE_ENABLE_CONST)
        SER_EzI2CSaveConfig();

    #elif (SER_SCB_MODE_SPI_CONST_CFG && SER_SPI_WAKE_ENABLE_CONST)
        SER_SpiSaveConfig();

    #elif (SER_SCB_MODE_UART_CONST_CFG && SER_UART_WAKE_ENABLE_CONST)
        SER_UartSaveConfig();

    #else

        SER_backup.enableState = (uint8) SER_GET_CTRL_ENABLED;

        if(0u != SER_backup.enableState)
        {
            SER_Stop();
        }

    #endif /* defined (SER_SCB_MODE_I2C_CONST_CFG) && (SER_I2C_WAKE_ENABLE_CONST) */

#endif /* (SER_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: SER_Wakeup
********************************************************************************
*
* Summary:
*  Prepares the component for the Active mode operation after exiting
*  Deep Sleep. The "Enable wakeup from Sleep Mode" option has an influence
*  on this function implementation.
*  This function should not be called after exiting Sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SER_Wakeup(void)
{
#if(SER_SCB_MODE_UNCONFIG_CONST_CFG)

    if(SER_SCB_WAKE_ENABLE_CHECK)
    {
        if(SER_SCB_MODE_I2C_RUNTM_CFG)
        {
            SER_I2CRestoreConfig();
        }
        else if(SER_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            SER_EzI2CRestoreConfig();
        }
    #if(!SER_CY_SCBIP_V1)
        else if(SER_SCB_MODE_SPI_RUNTM_CFG)
        {
            SER_SpiRestoreConfig();
        }
        else if(SER_SCB_MODE_UART_RUNTM_CFG)
        {
            SER_UartRestoreConfig();
        }
    #endif /* (!SER_CY_SCBIP_V1) */
        else
        {
            /* Unknown mode */
        }
    }
    else
    {
        if(0u != SER_backup.enableState)
        {
            SER_Enable();
        }
    }

#else

    #if (SER_SCB_MODE_I2C_CONST_CFG  && SER_I2C_WAKE_ENABLE_CONST)
        SER_I2CRestoreConfig();

    #elif (SER_SCB_MODE_EZI2C_CONST_CFG && SER_EZI2C_WAKE_ENABLE_CONST)
        SER_EzI2CRestoreConfig();

    #elif (SER_SCB_MODE_SPI_CONST_CFG && SER_SPI_WAKE_ENABLE_CONST)
        SER_SpiRestoreConfig();

    #elif (SER_SCB_MODE_UART_CONST_CFG && SER_UART_WAKE_ENABLE_CONST)
        SER_UartRestoreConfig();

    #else

        if(0u != SER_backup.enableState)
        {
            SER_Enable();
        }

    #endif /* (SER_I2C_WAKE_ENABLE_CONST) */

#endif /* (SER_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/* [] END OF FILE */
