/*******************************************************************************
* File Name: SER_SPI_UART_INT.c
* Version 3.0
*
* Description:
*  This file provides the source code to the Interrupt Service Routine for
*  the SCB Component in SPI and UART modes.
*
* Note:
*
********************************************************************************
* Copyright 2013-2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SER_PVT.h"
#include "SER_SPI_UART_PVT.h"
#include "cyapicallbacks.h"

#if (SER_SCB_IRQ_INTERNAL)
/*******************************************************************************
* Function Name: SER_SPI_UART_ISR
********************************************************************************
*
* Summary:
*  Handles the Interrupt Service Routine for the SCB SPI or UART modes.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
CY_ISR(SER_SPI_UART_ISR)
{
#if (SER_INTERNAL_RX_SW_BUFFER_CONST)
    uint32 locHead;
#endif /* (SER_INTERNAL_RX_SW_BUFFER_CONST) */

#if (SER_INTERNAL_TX_SW_BUFFER_CONST)
    uint32 locTail;
#endif /* (SER_INTERNAL_TX_SW_BUFFER_CONST) */

#ifdef SER_SPI_UART_ISR_ENTRY_CALLBACK
    SER_SPI_UART_ISR_EntryCallback();
#endif /* SER_SPI_UART_ISR_ENTRY_CALLBACK */

    if (NULL != SER_customIntrHandler)
    {
        SER_customIntrHandler();
    }

    #if (SER_CHECK_SPI_WAKE_ENABLE)
    {
        /* Clear SPI wakeup source */
        SER_ClearSpiExtClkInterruptSource(SER_INTR_SPI_EC_WAKE_UP);
    }
    #endif

    #if (SER_CHECK_RX_SW_BUFFER)
    {
        if (SER_CHECK_INTR_RX_MASKED(SER_INTR_RX_NOT_EMPTY))
        {
            do
            {
                /* Move local head index */
                locHead = (SER_rxBufferHead + 1u);

                /* Adjust local head index */
                if (SER_INTERNAL_RX_BUFFER_SIZE == locHead)
                {
                    locHead = 0u;
                }

                if (locHead == SER_rxBufferTail)
                {
                    #if (SER_CHECK_UART_RTS_CONTROL_FLOW)
                    {
                        /* There is no space in the software buffer - disable the
                        * RX Not Empty interrupt source. The data elements are
                        * still being received into the RX FIFO until the RTS signal
                        * stops the transmitter. After the data element is read from the
                        * buffer, the RX Not Empty interrupt source is enabled to
                        * move the next data element in the software buffer.
                        */
                        SER_INTR_RX_MASK_REG &= ~SER_INTR_RX_NOT_EMPTY;
                        break;
                    }
                    #else
                    {
                        /* Overflow: through away received data element */
                        (void) SER_RX_FIFO_RD_REG;
                        SER_rxBufferOverflow = (uint8) SER_INTR_RX_OVERFLOW;
                    }
                    #endif
                }
                else
                {
                    /* Store received data */
                    SER_PutWordInRxBuffer(locHead, SER_RX_FIFO_RD_REG);

                    /* Move head index */
                    SER_rxBufferHead = locHead;
                }
            }
            while(0u != SER_GET_RX_FIFO_ENTRIES);

            SER_ClearRxInterruptSource(SER_INTR_RX_NOT_EMPTY);
        }
    }
    #endif


    #if (SER_CHECK_TX_SW_BUFFER)
    {
        if (SER_CHECK_INTR_TX_MASKED(SER_INTR_TX_NOT_FULL))
        {
            do
            {
                /* Check for room in TX software buffer */
                if (SER_txBufferHead != SER_txBufferTail)
                {
                    /* Move local tail index */
                    locTail = (SER_txBufferTail + 1u);

                    /* Adjust local tail index */
                    if (SER_TX_BUFFER_SIZE == locTail)
                    {
                        locTail = 0u;
                    }

                    /* Put data into TX FIFO */
                    SER_TX_FIFO_WR_REG = SER_GetWordFromTxBuffer(locTail);

                    /* Move tail index */
                    SER_txBufferTail = locTail;
                }
                else
                {
                    /* TX software buffer is empty: complete transfer */
                    SER_DISABLE_INTR_TX(SER_INTR_TX_NOT_FULL);
                    break;
                }
            }
            while (SER_SPI_UART_FIFO_SIZE != SER_GET_TX_FIFO_ENTRIES);

            SER_ClearTxInterruptSource(SER_INTR_TX_NOT_FULL);
        }
    }
    #endif
    
#ifdef SER_SPI_UART_ISR_EXIT_CALLBACK
    SER_SPI_UART_ISR_ExitCallback();
#endif /* SER_SPI_UART_ISR_EXIT_CALLBACK */
    
}

#endif /* (SER_SCB_IRQ_INTERNAL) */


/* [] END OF FILE */
